<?php

namespace Smirik\PropelAdminBundle\Column;

use Smirik\PropelAdminBundle\Action\Action;
use Smirik\PropelAdminBundle\Action\ActionCollection;

class DataGrid implements \IteratorAggregate
{

    /**
     * List of all columns
     * @var array $columns
     */
    protected $columns;

    /**
     * List of actions with columns
     * @var $actions
     */
    protected $actions;

    /**
     * All native actions(i.e. edit, delete etc) are stored and displayed separately
     * @var ActionCollection
     */
    protected $nativeActions;

    protected $templates = array(
        'form' => array(
            'edit'   => 'SmirikPropelAdminBundle:Admin/Form:edit.html.twig',
            'new'    => 'SmirikPropelAdminBundle:Admin/Form:new.html.twig',
            'fields' => 'SmirikPropelAdminBundle:Admin/Form:fields.html.twig',
        ),
        'list' => array(
            'mass_actions'   => 'SmirikPropelAdminBundle:Admin/List:mass_actions.html.twig',
            'paginate'       => 'SmirikPropelAdminBundle:Admin/List:paginate.html.twig',
            'single_actions' => 'SmirikPropelAdminBundle:Admin/List:single_actions.html.twig',
            'table_filters'  => 'SmirikPropelAdminBundle:Admin/List:table_filters.html.twig',
            'table_header'   => 'SmirikPropelAdminBundle:Admin/List:table_header.html.twig',
        ),
        'index'         => 'SmirikPropelAdminBundle:Admin:index.html.twig',
        'index_content' => 'SmirikPropelAdminBundle:Admin:index_content.html.twig',
        'row'           => 'SmirikPropelAdminBundle:Admin:row.html.twig',
    );

    /**
     * Show options
     */
    protected $is_sortable = false;
    protected $is_filterable = false;

    protected $has_avalanche = false;

    protected $limit = 15;

    public function __construct()
    {
        $this->actions = new ActionCollection();
        $this->columns = new ColumnCollection();
        $this->nativeActions = new ActionCollection();
    }

    public function setupAvalanche()
    {
        $this->has_avalanche = true;
    }
    
    public function hasAvalanche()
    {
        return $this->has_avalanche;
    }

    /**
     * Loads & setup actions & columns config. 
     * @param array $config
     * @param Smirik\PropelAdminBundle\Manager\ColumnManager $column_manager
     * @param Smirik\PropelAdminBundle\Manager\ActionManager $action_manager
     * @return void
     */
    public function load($config, $column_manager, $action_manager)
    {
        /**
         * Load columns & actions
         */
        $columns = array();
        foreach ($config['columns'] as $key => $column) {
            $columns[$key] = $column_manager->create($column);
        }
        $this->columns->setColumns($columns);

        $actions = array();
        $nativeActions = array();
        foreach ($config['actions'] as $key => $action) {

            /** @var Action $temp */
            $temp = $action_manager->create($action);

            $temp->isNative() ? $nativeActions[$key] = $temp : $actions[$key] = $temp;
        }
        $this->actions->setActions($actions);
        $this->nativeActions->setActions($nativeActions);

        /**
         * Merge custom templates
         */
        if (isset($config['templates'])) {
            foreach ($this->templates as $key => $array)
            {
                if (isset($config['templates'][$key]))
                {
                    if (is_array($array))
                    {
                        $this->templates[$key] = $config['templates'][$key] + $this->templates[$key];
                    } else
                    {
                        $this->templates[$key] = $config['templates'][$key];
                    }
                }
            }
        }
        
    }

    /**
     * Get template related to relative name
     * @param string $name
     * @throws \Exception
     * @return string
     */
    public function template($name)
    {
        /**
         * @todo refactoring
         */
        $tmp = explode('.', $name);
        $num = count($tmp);
        if ($num == 1)
        {
            return $this->templates[$name];
        } elseif ($num == 2)
        {
            if (!isset($this->templates[$tmp[0]][$tmp[1]]))
            {
                throw new \Exception('Template not found');
            }
            return $this->templates[$tmp[0]][$tmp[1]];
        }
    }
    
    public function getIterator()
    {
        return new \ArrayIterator($this->columns);
    }

    public function getColumns()
    {
        return $this->columns;
    }

    public function getActions()
    {
        return $this->actions;
    }

    public function setLimit($limit)
    {
        $this->limit = $limit;
    }

    public function getLimit()
    {
        return $this->limit;
    }

    /**
     * @return ActionCollection
     */
    public function getNativeActions()
    {
        return $this->nativeActions;
    }

}